import sys
from mglutil.regression import testplus

def setUpSuite():
    from geomutils import efitlib
def test_lib():
    from geomutils import efitlib
    ellipse = efitlib.ellipsoid()
    ellipseInfo = efitlib.efit_info()
    from Data.crn_coords import crn_coords
    ell_scale = 1.0  # 0.0 or belove defaults to 1.0
    cov_scale= 1.75  # 0.0 or smaller defaults to 1.75
    efitlib.fitEllipse(crn_coords, ell_scale, cov_scale, ellipseInfo, ellipse)

    print 'center:', ellipse.getPosition()
    print 'scaling:', ellipse.getAxis()
    print 'orientation', ellipse.getOrientation()

    ellipse = efitlib.ellipsoid()
    ellipseInfo = efitlib.efit_info()
    from Data.cv_coords import cv_coords
    efitlib.fitEllipse(cv_coords, ell_scale, cov_scale, ellipseInfo, ellipse)

    ## print 'weightflag:', ellipseInfo.weightflag
    ## print 'covarflag:', ellipseInfo.covarflag
    ## print 'volumeflag:', ellipseInfo.volumeflag
    ## print 'matrixflag:', ellipseInfo.matrixflag
    ## print 'nocenterflag:', ellipseInfo.nocenterflag
    ## print 'noscaleflag:', ellipseInfo.noscaleflag
    ## print 'nosortflag:', ellipseInfo.nosortflag
    ## print 'count:', ellipseInfo.count
    ## print 'cov_scale:', ellipseInfo.cov_scale
    ## print 'ell_scale:', ellipseInfo.ell_scale

    print 'center:', ellipse.getPosition()
    print 'scaling:', ellipse.getAxis()
    print 'orientation', ellipse.getOrientation()

    vec = [.28, 0.89, 1.5]
    vec1=efitlib.vec_normalize(vec)
    print "vector:", vec
    print "normalized vector:", vec1
    

if __name__ == '__main__':
    testplus.chdir()
    harness = testplus.TestHarness( __name__,
                                connect = (setUpSuite, (), {}),
                                funs = testplus.testcollect( globals()),
                                )
    print harness
    sys.exit( len( harness))



from math import sqrt

def triangleArea(p1, p2, p3):
    """Compute the surface area of a triangle.
"""
    x1,y1,z1 = p1
    x2,y2,z2 = p2
    x3,y3,z3 = p3
    dx, dy, dz = x1-x2, y1-y2, z1-z2
    a = sqrt( dx*dx + dy*dy + dz*dz )
    dx, dy, dz = x2-x3, y2-y3, z2-z3
    b = sqrt( dx*dx + dy*dy + dz*dz )
    dx, dy, dz = x1-x3, y1-y3, z1-z3
    c = sqrt( dx*dx + dy*dy + dz*dz )
    s = .5*(a+b+c)
    area = s*(s-a)*(s-b)*(s-c)
    if area <= 0.:
#        print "area = %f for triangles: " % area, p1, p2, p3
        return 0.
    return sqrt(area)

def meshVolume(verts, norm, tri):
    """Compute the Volume of a mesh specified by vertices, their normals, and
indices of triangular faces
"""
    # TEST
    zeronorms = []
    for i, n in enumerate(norm):
        #if n == [0., 0., 0.] or n == (0., 0., 0.):
	if n[0] == 0 and n[1] == 0 and n[2] == 0:
	    #print "normal %d is zero!" % i, n
	    zeronorms.append(i)
    #print "in meshVolume, zeronorms length: ", len(zeronorms), "normals length:", len(norm)
    # Initialize
    volSum = 0.0
    oneThird = 1./3.
    
    # Compute face normals
    trinorm = []
    for t in tri:
        n1 = norm[t[0]]
        n2 = norm[t[1]]
        n3 = norm[t[2]]
	tn = [ (n1[0]+n2[0]+n3[0])*oneThird,
               (n1[1]+n2[1]+n3[1])*oneThird,
               (n1[2]+n2[2]+n3[2])*oneThird ]
	trinorm.append(tn)
    # print trinorm	# TEST

    # Compute volume
    for t,tn in zip(tri, trinorm):
        s1 = verts[t[0]]
        s2 = verts[t[1]]
        s3 = verts[t[2]]
        area = triangleArea(s1,s2,s3)

	g = [ (s1[0]+s2[0]+s3[0])*oneThird,
              (s1[1]+s2[1]+s3[1])*oneThird,
              (s1[2]+s2[2]+s3[2])*oneThird ]
        volSum += (g[0]*tn[0] + g[1]*tn[1] + g[2]*tn[2])*area
    return volSum*oneThird


def findComponents(verts, faces, normals=None, returnOption=0):
    	"""find the components of a geometry.
normals are normals of verts not faces.
returnOptiont: 	component return option
	= 0:		return all components
	= 1:		return all outside surfaces (volume > 0; normals must be given)
The code is based on the Vision node ConnectedComponents.
"""
        fdict = {} 
        vdict = {} #dictionary with key - vertex index,
                   #value - list of face indices in which the vertex is found

        flag1 = True; flag2 = True
        newfaces = [] 
	newverts = []
	if normals is not None:
	    newnorms = []
        while flag2:
            for i, fs in enumerate(faces):
                for v in fs:
                    if not vdict.has_key(v):
                        vdict[v] = [i]
                    else:
                        vdict[v].append(i)
                fdict[i] = fs
            Vco = faces[0][:]
            newfaces1 = []
	    newverts1 = []
	    if normals is not None:
	        newnorms1 = []
            vertinds = {} # keys - vertex indices from the input verts list
                          # values - new vertex indices of current surface
            vcount = 0
            # find a surface
            while flag1:
                _Vco = []
                flag1 = False
                # find all vertices that share the same triangles with the vertices in Vco.
                for vert in Vco:
                    vfs = vdict[vert]
                    for i in vfs:
                        if fdict.has_key(i):
                            flag1 = True
                            fs = fdict.pop(i)

                            fsnew = [] # remapped face (with new vertex idices)
                            for v in fs:
                                if v not in Vco: 
                                    if v not in _Vco:
                                        _Vco.append(v)
                                if not  vertinds.has_key(v):
                                    vertinds[v] = vcount
                                    newverts1.append(verts[v])
				    if normals is not None:
				        newnorms1.append(normals[v])
                                    fsnew.append(vcount) 
                                    vcount = vcount + 1
                                else:
                                    fsnew.append(vertinds[v])
                            newfaces1.append(fsnew) # add found triangle to the list of triangles of current surface

                Vco  = _Vco
            newfaces.append( newfaces1 )
            newverts.append( newverts1 )
	    if normals is not None:
	        newnorms.append( newnorms1 )
		
            if  len(fdict):
                faces = fdict.values()
                fdict = {}
                vdict = {}
                flag1 = True
            else: flag2 = False


	# return all surfaces
	if returnOption == 0:
	    if normals is not None:
	        return newverts, newfaces, newnorms
	    else:
	        return newverts, newfaces
	    
	# return only outside surfaces
	outverts = []
	outfaces = []
	outnorms = []
        for i in range( len(newfaces) ):
	    Nvert = len(outverts)
	    volume = meshVolume(newverts[i], newnorms[i], newfaces[i])
	    print i, len(newverts[i]), len(newfaces[i]), volume	# TEST
	    if volume > 0:
	        outverts += newverts[i]
		outnorms += newnorms[i]
		# update face vertex indices to reflect lengthened vertices
		newfaces_i_mod = []
		for v1, v2, v3 in newfaces[i]:
		    newfaces_i_mod.append( [v1+Nvert, v2+Nvert, v3+Nvert] )
		outfaces += newfaces_i_mod

	return outverts, outfaces, outnorms




from distutils.core import setup, Extension
from distutils.cmd import Command
from distutils.command.install_data import install_data
from distutils.command.build import build
from distutils.command.sdist import sdist
from glob import glob
from distutils.command.build_ext import build_ext

import sys, os
from os import path

pack_name = "geomutils"
platform = sys.platform
########################################################################
# Had to overwrite the run method of the install_data to install the data file
# in the package instead of a particular data directory
########################################################################

class modified_install_data(install_data):

    def run(self):
        install_cmd = self.get_finalized_command('install')
        self.install_dir = getattr(install_cmd, 'install_lib')
        return install_data.run(self)

#####################################################################
# Had to overwrite the prune_file_list method of sdist to not
# remove automatically the RCS/CVS directory from the distribution.
####################################################################

class modified_sdist(sdist):
    def prune_file_list(self):

        build = self.get_finalized_command('build')
        base_dir = self.distribution.get_fullname()
        self.filelist.exclude_pattern(None, prefix=build.build_base)
        self.filelist.exclude_pattern(None, prefix=base_dir)

########################################################################
# Had to overwrite the sub_commands list of the build command so that
# the build_py is called after build_clib and build_ext. This way
# when building the extension the .py is generated before the other
# py module are. This avoid to have to first call build then install.
########################################################################
        

class modified_build(build):
    sub_commands = [('build_clib',    build.has_c_libraries),
                    ('build_ext',     build.has_ext_modules),
                    ('build_py',      build.has_pure_modules),
                    ('build_scripts', build.has_scripts),
                   ]
    
########################################################################

# list of the python packages to be included in this distribution.
# sdist doesn't go recursively into subpackages so they need to be
# explicitaly listed.
# From these packages only the python modules will be taken
packages = ['geomutils', 'geomutils.Tests', 'geomutils.Tests.Data']

# list of the python modules not part of a package. Give the path and the
# filename without the extension. i.e you want to add the
# test.py module which is located in MyPack/Tests/ you give
# 'MyPack/Tests/test'
py_modules = []

# list of the Macros
definemacros = []
if platform == 'sunos5':
    definemacros.append(('sqrtf', 'sqrt'))
if platform == 'win32':
    definemacros.append(('sqrtf', 'sqrt'))
    definemacros.append(('NO_DRAND48', None))
    definemacros.append(('random', 'rand'))
    definemacros.append(('srandom', 'srand'))

class modified_build_ext (build_ext):
    def spawn(self, cmd, search_path=1, level=1):
        cmd.insert(-1, "-outdir")
        cmd.insert(-1, pack_name)
        print "command: ", cmd
        apply(build_ext.spawn, (self, cmd), {})

    def build_extension(self, ext):
        comp_modif = 0
        swig_cpp = self.swig_cpp
        language = self.compiler.detect_language(ext.sources)
        print "language: ", language
        if language == "c++":
            if platform != "win32":
                comp = self.compiler.compiler[0]
                if comp == "gcc":
                    self.compiler.compiler[0] = 'g++'
                    self.compiler.compiler_so[0] = 'g++'
                    self.compiler.linker_so[0] = 'g++'
                elif comp == "cc":
                    self.compiler.compiler[0] = 'CC'
                    self.compiler.compiler_so[0] = 'CC'
                    self.compiler.linker_so[0] = 'CC'
            self.swig_cpp = 1
            comp_modif = 1
        build_ext.build_extension(self, ext)
        if comp_modif:
             if platform != "win32":
                 if comp == "gcc":
                     self.compiler.compiler[0] = 'gcc'
                     self.compiler.compiler_so[0] = 'gcc'
                     self.compiler.linker_so[0] = 'gcc'
                 elif comp == "cc":
                     self.compiler.compiler[0] = 'cc'
                     self.compiler.compiler_so[0] = 'cc'
                     self.compiler.linker_so[0] = 'cc'
             self.swig_cpp = swig_cpp


if platform == 'win32':
    libs =[]
elif platform == 'darwin':
    libs = ["mx"]
else:
    libs=["m"]

efit_srcfiles = []
for f in ["efit.c", "efit_aux.c", "efit_io.c", "vec.c"]:
    efit_srcfiles.append(path.join("src", "efit", f))
efit_srcfiles.append(path.join("geomutils", "efitlib.i"))
import numpy
numpy_include =  numpy.get_include()
efit_ext = Extension ("_efitlib", efit_srcfiles,
                                       include_dirs = [path.join("src", "efit"),numpy_include],
                                       define_macros = definemacros,
                                       libraries = libs
                                       )
geomalg_libs = []
geomalg_srcfiles = []
geomalg_comp_args = []
geomalg_link_args = []
for f in ["point.cpp", "Vector.cpp", "distance.cpp", "intersections.cpp",
          "polygonArea.cpp", "boundingContainers.cpp",
          "objfile.cpp"
          ]:
    geomalg_srcfiles.append(path.join("src", "geomAlgorithms", f))
geomalg_srcfiles.append(path.join("geomutils", "geomalgorithms.i"))
if platform == "sunos5":
    geomalg_libs.extend(["Crun", "Cstd"])
elif platform == "irix6":
    geomalg_comp_args.append( "-LANG:std" )
    geomalg_link_args.append("-LANG:std")
elif platform == "win32":
     geomalg_comp_args.extend(["/MT"])

geomalg_ext = Extension("_geomalgorithms", geomalg_srcfiles,
                        include_dirs = [path.join("src", "geomAlgorithms"),
                                        numpy_include],
                        define_macros = [],
                        libraries = geomalg_libs,
                        extra_compile_args = geomalg_comp_args,
                        extra_link_args=geomalg_link_args
                        )


sdf_ext = Extension ("_sdflib", [path.join("geomutils", "sdf.i")],
                                       include_dirs = [path.join("src", "efit"),
                                                       numpy_include],
                                       define_macros = [],
                                       libraries = []
                                       )            
########################################################################
# description of what is going to be included in the distribution and
# installed.

try:   
    from version import VERSION
except:
    VERSION = "1.0"
    
if platform == "win32":
    all_modules =  [efit_ext, geomalg_ext]
else:
     all_modules =  [efit_ext, geomalg_ext, sdf_ext]    
dist = setup(name=pack_name,
             version=VERSION,
             description = "geomutils library",
             author = 'Molecular Graphics Laboratory',
             author_email = 'mgltools@scripps.edu',
             download_url = 'http://www.scripps.edu/~sanner/software/packager.html',
             url = 'http://www.scripps.edu/~sanner/software/index.html',
             packages = packages,
             py_modules = py_modules,
             cmdclass = {'build': modified_build,
                         'build_ext': modified_build_ext,
                         'sdist' : modified_sdist, 
                      },
             ext_package = "geomutils",
             ext_modules = all_modules#[efit_ext, geomalg_ext, sdf_ext],
             )
                             
# Remove the .py and wrap.c files generated by swig from the
# MyPack directory
##  import os
##  pyName = './%s/%slib.py'%(pack_name, pack_name)
##  if os.path.exists(pyName):
##      os.system("rm -f %s"%pyName)
##      print "Removing %s generated by swig"%pyName
##  wrapName = './%s/%slib_wrap.c'%(pack_name, pack_name)
##  if os.path.exists(wrapName):
##      os.system("rm -f %s"%wrapName)
##      print "Removing %s generated by swig"%wrapName


